import gocept.directory2.admin.testing
import gocept.directory2.admin.domain.testing


class TestDomainUserCRUD(gocept.directory2.admin.testing.BrowserTestCase,
                         gocept.directory2.admin.domain.testing.DomainMixin):

    def test_user_list_contains_list_of_domains_with_url(self):
        self.create_domain()
        browser = self.browser
        browser.open('http://localhost/domain/user/list')
        self.assertEqual('http://localhost/domain/user/100/',
                         browser.getLink('ctheune').url)
        self.assertEqual('http://localhost/domain/1',
                         browser.getLink('flyingcircus.io').url)

    def test_user_list_does_not_contain_subdomains(self):
        self.create_domains_with_subdomains()
        browser = self.browser
        browser.open('http://localhost/domain/user/list')
        self.assertNotIn('www.flyingcircus.io', browser.contents)

    def test_add_user_suggests_uid_as_its_unique_but_not_auto_increment(self):
        self.create_domain()
        browser = self.browser
        browser.open('http://localhost/domain/user/add')
        self.assertEqual('101', browser.getControl('UID').value)
        # you may change the number
        browser.getControl('UID').value = '10'
        browser.getControl('Name').value = 'myusername'
        browser.getControl(name='add').click()
        self.assertIn('added successfully', browser.contents)
        browser.open('http://localhost/domain/user/add')
        self.assertEqual('101', browser.getControl('UID').value)
        browser.getControl('Name').value = 'myusername2'
        browser.getControl(name='add').click()
        self.assertIn('added successfully', browser.contents)
        browser.open('http://localhost/domain/user/add')
        self.assertEqual('102', browser.getControl('UID').value)

    def test_domain_list_for_user(self):
        self.create_domains_with_subdomains()
        browser = self.browser
        browser.open('http://localhost/domain/user/100/list_domains')
        self.assertIn('flyingcircus.io', browser.contents)
        self.assertNotIn('www.flyingcircus.io', browser.contents)

    def test_add_domain_in_context_of_user_preselects_user(self):
        from gocept.directory2.domain import Domain
        self.create_domain()
        browser = self.browser
        browser.open('http://localhost/domain/user/100/add_domain')
        browser.getControl('Domainname').value = 'testing.com'
        browser.getControl(name='add').click()
        self.assertIn('added successfully', browser.contents)
        added = Domain.query().filter_by(id=2).all()[0]
        self.assertEqual('testing.com', added.name)
        self.assertEqual(100, added.user.id)
