from ..location import Location
from .schema import is_ascii
from .view import View, Data, Action
from colander import SchemaNode, String
from pyramid.view import view_config
import colander
import gocept.directory2.admin.form


@view_config(route_name='location_list',
             renderer='templates/location_list.pt')
class ListLocations(View):
    """List known locations."""
    title = 'Locations'
    actions = [Action('Add location', 'icon-plus', 'location_add')]

    @property
    def locations(self):
        for x in Location.query():
            yield Data(title=x.name,
                       url=self.request.route_url('location_view', id=x.id))


class LocationViews(object):

    @property
    def title(self):
        return self.context.name

    @property
    def actions(self):
        yield Action('Edit', 'icon-edit', 'location_edit', id=self.context.id)
        yield Action('Delete', 'icon-remove', 'location_confirm_delete',
                     id=self.context.id)

    @property
    def tabs(self):
        return [
            Action('Info', '', 'location_view', id=self.context.id),
            Action('Racks', '', 'location_racks', id=self.context.id)
        ]


@view_config(route_name='location_view',
             renderer='templates/location_view.pt')
class ViewLocation(LocationViews, View):
    pass


@view_config(route_name='location_racks',
             renderer='templates/location_racks.pt')
class ViewLocationRacks(LocationViews, View):
    pass


class Schema(colander.Schema):
    """Schema to add a location."""

    name = SchemaNode(String())
    abbreviation = SchemaNode(
        String(), title='Short name / Identifier', validator=is_ascii)


@view_config(route_name='location_add')
class Add(gocept.directory2.admin.form.Add):
    """Add a new location."""

    title = 'Add location'
    schema = Schema()
    factory = Location
    next_route = 'location_list'


@view_config(route_name='location_edit')
class Edit(gocept.directory2.admin.form.Edit):

    title = 'Edit location'
    schema = Schema()
    next_route = 'location_view'


@view_config(route_name='location_confirm_delete')
class ConfirmDelete(gocept.directory2.admin.form.ConfirmDelete):
    """Confirm deletion of location."""

    delete_route = 'location_delete'

    @property
    def subject(self):
        return 'location "%s"' % self.context.name


@view_config(route_name='location_delete')
class Delete(gocept.directory2.admin.form.Delete):
    """Actually delete location."""

    next_route = 'location_list'
