from ..location import Location
from ..machine import Node, PhysicalMachine, VirtualMachine
from ..resourcegroup import ResourceGroup
from .schema import Relation, ReadOnlyNode
from .view import View, Action
from .widget import RequiredSelectWidget
from colander import SchemaNode, String
from pyramid.view import view_config
import colander
import gocept.directory2.admin.form


@view_config(route_name='machine_list',
             renderer='templates/machine/list.pt')
class ListMachines(View):
    """List known machines."""
    title = 'Machines'
    actions = [
        Action('Add virtual machine', 'icon-plus', 'machine_add_virtual'),
        Action('Add physical machine', 'icon-plus', 'machine_add_physical'),
    ]

    @property
    def machines(self):
        return Node.query()


@view_config(route_name='home',
             renderer='templates/machine/list.pt')
class HomePage(ListMachines):
    pass


@view_config(route_name='machine_view_physical',
             renderer='templates/machine/view_physical.pt')
class ViewPhysical(View):

    @property
    def title(self):
        return self.context.name

    @property
    def actions(self):
        yield Action('Edit', 'icon-edit',
                     'machine_edit_%s' % self.context.class_,
                     id=self.context.id)
        yield Action('Delete', 'icon-remove', 'machine_confirm_delete',
                     id=self.context.id)


@view_config(route_name='machine_view_virtual',
             renderer='templates/machine/view_virtual.pt')
class ViewVirtual(ViewPhysical):
    pass


class AddSchema(colander.Schema):
    """Schema to add a physical machine."""

    # XXX refactor
    @colander.deferred
    def location_widget(node, kw):
        return RequiredSelectWidget(
            values=[(x.id, x.name) for x in Location.query()])

    @colander.deferred
    def resourcegroup_widget(node, kw):
        return RequiredSelectWidget(
            values=[(x.id, x.name) for x in ResourceGroup.query()])

    name = SchemaNode(String(), title='Hostname')
    location = SchemaNode(
        Relation(Location), widget=location_widget)
    service_description = SchemaNode(
        String(), missing='',
        description='This node provides the following services: '
        '(for customer information)')
    resource_group = SchemaNode(
        Relation(ResourceGroup), widget=resourcegroup_widget)


class AddVirtualSchema(AddSchema):

    @colander.deferred
    def kvm_host_widget(node, kw):
        return RequiredSelectWidget(
            values=[(x.id, x.name) for x in PhysicalMachine.query()])

    kvm_host = SchemaNode(
        Relation(PhysicalMachine), widget=kvm_host_widget, missing=None)


@view_config(route_name='machine_add_physical')
class AddPhysical(gocept.directory2.admin.form.Add):
    """Add a new physical machine."""

    title = 'Add physical machine'
    schema = AddSchema()
    factory = PhysicalMachine
    next_route = 'machine_list'


@view_config(route_name='machine_add_virtual')
class AddVirtual(gocept.directory2.admin.form.Add):
    """Add a new virtual machine."""

    title = 'Add virtual machine'
    schema = AddVirtualSchema()
    factory = VirtualMachine
    next_route = 'machine_list'


class EditSchema(AddSchema):
    """Edit a physical machine."""

    name = ReadOnlyNode(String(), title='Hostname')


class EditVirtualSchema(EditSchema, AddVirtualSchema):
    pass


@view_config(route_name='machine_edit_physical')
class EditPhysical(gocept.directory2.admin.form.Edit):

    title = 'Edit machine'
    schema = EditSchema()
    next_route = 'machine_view_physical'


@view_config(route_name='machine_edit_virtual')
class EditVirtual(gocept.directory2.admin.form.Edit):

    title = 'Edit machine'
    schema = EditVirtualSchema()
    next_route = 'machine_view_virtual'


@view_config(route_name='machine_confirm_delete')
class ConfirmDelete(gocept.directory2.admin.form.ConfirmDelete):
    """Confirm deletion of physical machine."""

    delete_route = 'machine_delete'

    @property
    def subject(self):
        return 'physical machine "%s"' % self.context.name


@view_config(route_name='machine_delete')
class Delete(gocept.directory2.admin.form.Delete):
    """Actually delete physical machine."""

    next_route = 'machine_list'
