from ..permission import Permission
from .schema import ReadOnlyNode
from .view import View, Data, Action
from colander import SchemaNode, Int, String
from deform.widget import TextAreaWidget
from pyramid.view import view_config
import colander
import gocept.directory2.admin.form


@view_config(route_name='permission_list',
             renderer='templates/permission_list.pt')
class ListPermissions(View):
    """List known permissions."""

    title = 'Permissions'
    actions = [Action('Add permission', 'icon-plus', 'permission_add')]

    @property
    def permissions(self):
        for permission in Permission.query():
            yield Data(
                name=permission.name,
                id_number=permission.id_number,
                description=permission.description,
                url=self.request.route_url(
                    'permission_view', id=permission.id))


@view_config(route_name='permission_view',
             renderer='templates/permission_view.pt')
class ViewPermission(View):

    @property
    def title(self):
        return u'Permission "{}"'.format(self.context.name)

    @property
    def actions(self):
        yield Action(
            'Edit', 'icon-edit', 'permission_edit', id=self.context.id)
        yield Action(
            'Delete', 'icon-remove', 'permission_confirm_delete',
            id=self.context.id)


class AddSchema(colander.Schema):
    """Schema to add a permission."""

    id_number = SchemaNode(Int(), title=u'Numerical id')
    name = SchemaNode(String())
    description = SchemaNode(String(), missing='', widget=TextAreaWidget())


@view_config(route_name='permission_add')
class Add(gocept.directory2.admin.form.Add):
    """Add a new permission."""

    title = 'Add permission'
    schema = AddSchema()
    factory = Permission
    next_route = 'permission_list'


class EditSchema(AddSchema):

    name = ReadOnlyNode(String())


@view_config(route_name='permission_edit')
class Edit(gocept.directory2.admin.form.Edit):

    title = 'Edit permission'
    schema = EditSchema()
    next_route = 'permission_view'


@view_config(route_name='permission_confirm_delete')
class ConfirmDelete(gocept.directory2.admin.form.ConfirmDelete):
    """Confirm deletion of permission."""

    delete_route = 'permission_delete'

    @property
    def subject(self):
        return 'permission "%s"' % self.context.name


@view_config(route_name='permission_delete')
class Delete(gocept.directory2.admin.form.Delete):
    """Actually delete permission."""

    next_route = 'permission_list'
