import gocept.directory2.admin.resources
import pyramid.renderers


class OwrapSelectionPredicate(object):
    """To use an own owrap for a view, pass the owrap kw
    to the view_config decorator, containing the path to
    the template used as owrap.
    To explicitly disable owrap, set it to False.
    """

    def __init__(self, owrap, config):
        self.owrap = owrap

    def text(self):
        return 'owrap = {}'.format(self.owrap)

    phash = text

    def __call__(self, context, request):
        request['owrap'] = self.owrap
        return True


def owrap_tween_factory(handler, registry):
    def owrap_tween(request):
        response = handler(request)

        if not hasattr(request, 'content_view'):
            # View not suiteable for this tween.
            return response

        if response.status_int != 200:
            return response

        if not (response.content_type and
                response.content_type.lower() in ['text/html', 'text/xml']):
            return response

        template = request.get('owrap', 'templates/main.pt')
        if not template:
            return response

        gocept.directory2.admin.resources.gocept_directory2.need()
        response.text = pyramid.renderers.render(
            template,
            {'content': unicode(response.body, 'utf-8'),
             'view': request.content_view,
             'static_url': lambda name: request.static_url(
                 'gocept.directory2.admin:resources/%s' % name),
             },
            request=request)
        return response
    return owrap_tween
