from gocept.directory2.location import Location
from gocept.directory2.machine import PhysicalMachine, VirtualMachine
import gocept.directory2.admin.testing
import gocept.testing.assertion


class TestMachineCRUD(gocept.directory2.admin.testing.BrowserTestCase,
                      gocept.testing.assertion.Ellipsis):
    """Testing ..machine.*"""

    def test_machine_list_displays_existing_machines(self):
        PhysicalMachine.create(
            name='foo',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        VirtualMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        b = self.browser
        b.open('http://localhost/machine/list')
        self.assertIn('foo', b.contents)
        self.assertIn('bar', b.contents)

    def test_add_physical_machine(self):
        Location.create(name='loc1')
        browser = self.browser
        browser.open('http://localhost/machine/list')
        browser.getLink('Add physical machine').click()
        browser.getControl('Hostname').value = 'bar'
        self.assertEqual(['- Select -', 'loc0', 'loc1'],
                         browser.getControl('Location').displayOptions)
        browser.getControl('Location').displayValue = ['loc0']
        browser.getControl('Resource Group').displayValue = ['rg0']
        browser.getControl(name='add').click()
        self.assertIn('added successfully', browser.contents)
        self.assertEqual(1, len(PhysicalMachine.query().all()))
        machine = PhysicalMachine.query().first()
        self.assertEqual('bar', machine.name)
        self.assertEqual('loc0', machine.location.name)

    def test_add_virtual_machine(self):
        browser = self.browser
        browser.open('http://localhost/machine/list')
        browser.getLink('Add virtual machine').click()
        browser.getControl('Hostname').value = 'bar'
        browser.getControl('Location').displayValue = ['loc0']
        browser.getControl('Resource Group').displayValue = ['rg0']
        browser.getControl(name='add').click()
        self.assertEqual(1, len(VirtualMachine.query().all()))

    def test_view_physical_machine(self):
        PhysicalMachine.create(
            name='baz',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        browser = self.browser
        browser.open('http://localhost/machine/list')
        browser.getLink('baz').click()
        self.assertEqual('http://localhost/machine/physical/1', browser.url)
        self.assertIn('baz', browser.contents)
        self.assertIn('loc0', browser.contents)

    def test_view_virtual_machine(self):
        VirtualMachine.create(
            name='baz',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        browser = self.browser
        browser.open('http://localhost/machine/list')
        browser.getLink('baz').click()
        self.assertEqual('http://localhost/machine/virtual/1', browser.url)
        self.assertIn('baz', browser.contents)
        self.assertIn('loc0', browser.contents)

    def test_edit_physical_machine(self):
        Location.create(name='loc1')
        PhysicalMachine.create(
            name='baz',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        browser = self.browser
        browser.open('http://localhost/machine/physical/1')
        browser.getLink('Edit').click()
        # Displays stored values:
        self.assertEqual(
            'http://localhost/machine/physical/1/edit', browser.url)
        # Hostname is a read only field
        with self.assertRaises(LookupError):
            browser.getControl('Hostname')
        self.assertIn('Hostname', browser.contents)
        self.assertIn('baz', browser.contents)
        self.assertEqual(['loc0'], browser.getControl('Location').displayValue)
        # Values can be changed:
        browser.getControl('Location').displayValue = ['loc1']
        browser.getControl(name='save').click()
        self.assertIn('Updated on', browser.contents)
        # Changed values get persisted:
        browser.getLink('Edit').click()
        self.assertEqual(['loc1'], browser.getControl('Location').displayValue)

    def test_edit_virtual_machine(self):
        Location.create(name='loc1')
        VirtualMachine.create(
            name='baz',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        browser = self.browser
        browser.open('http://localhost/machine/virtual/1')
        browser.getLink('Edit').click()
        browser.getControl('Location').displayValue = ['loc1']
        browser.getControl(name='save').click()
        self.assertIn('Updated on', browser.contents)
        browser.getLink('Edit').click()
        self.assertEqual(['loc1'], browser.getControl('Location').displayValue)

    def test_delete_machine(self):
        PhysicalMachine.create(
            name='qwe',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        browser = self.browser
        browser.open('http://localhost/machine/list')
        self.assertIn('qwe', browser.contents)
        browser.open('http://localhost/machine/physical/1')
        browser.getLink('Delete').click()
        self.assertIn(
            'Confirm deletion of physical machine', browser.contents)
        browser.getControl(name='delete').click()
        self.assertEqual('http://localhost/machine/list', browser.url)
        self.assertNotIn('qwe', browser.contents)

    def test_location_is_required_and_does_not_store_first_value(self):
        browser = self.browser
        browser.open('http://localhost/machine/add_physical')
        browser.getControl('Hostname').value = 'qux'
        browser.getControl(name='add').click()
        # The location field is the second one, sorry for this bad test but
        # there seems to be no better way to test this field:
        self.assertEllipsis(
            '...<span class="error" id="error-deformField2"> Required '
            '</span>...',
            browser.contents)
