from gocept.directory2.machine import PhysicalMachine
from gocept.directory2.resourcegroup import ResourceGroup
import gocept.directory2.admin.testing


class ResourceGroupCRUDTests(gocept.directory2.admin.testing.BrowserTestCase):
    """Testing ..resourcegroup.*"""

    def test_list_displays_existing_resourcegroups(self):
        ResourceGroup.create(name='foo')
        browser = self.browser
        browser.open('http://localhost/resourcegroup/list')
        self.assertIn('foo', browser.contents)

    def test_add_resourcegroup(self):
        browser = self.browser
        browser.open('http://localhost/resourcegroup/list')
        browser.getLink('Add resource group').click()
        # Checking usage of default values:
        self.assertEqual(['Europe/Berlin'],
                         browser.getControl('Timezone').displayValue)
        self.assertEqual(['22'], browser.getControl('Start hour').value)
        # Technical Contacts is a required field which cannot be entered
        # without JavaScript:
        browser.getControl(name='add').click()
        self.assertIn('Shorter than minimum length 1', browser.contents)

    def test_view_resourcegroup(self):
        ResourceGroup.create(name='baz', customer_no='12345',
                             description="Resource group for baz' biz")
        browser = self.browser
        browser.open('http://localhost/resourcegroup/list')
        browser.getLink('baz').click()
        self.assertEqual('http://localhost/resourcegroup/3', browser.url)
        self.assertIn('baz', browser.contents)
        self.assertIn('12345', browser.contents)
        self.assertIn("Resource group for baz' biz", browser.contents)

    def test_edit_resourcegroup(self):
        ResourceGroup.create(name='baz', description='RG of baz',
                             technical_contacts=['baz@example.com'])
        browser = self.browser
        browser.open('http://localhost/resourcegroup/list')
        browser.getLink('baz').click()
        browser.getLink('Edit').click()
        # Displays stored values:
        # Name is a read only field
        with self.assertRaises(LookupError):
            browser.getControl('Name')
        self.assertIn('Name', browser.contents)
        self.assertIn('baz', browser.contents)
        self.assertEqual('RG of baz', browser.getControl('Description').value)
        self.assertEqual(['22'], browser.getControl('Start hour').value)
        # Values can be changed:
        browser.getControl('Description').value = "baz' biz"
        browser.getControl(name='save').click()
        self.assertIn('Updated on', browser.contents)
        # Changed values got persisted:
        browser.getLink('Edit').click()
        self.assertEqual("baz' biz", browser.getControl('Description').value)

    def test_delete_resourcegroup(self):
        ResourceGroup.create(name='qwe')
        browser = self.browser
        browser.open('http://localhost/resourcegroup/list')
        browser.getLink('qwe').click()
        browser.getLink('Delete').click()
        self.assertIn(
            'Confirm deletion of resource group', browser.contents)
        browser.getControl(name='delete').click()
        self.assertEqual('http://localhost/resourcegroup/list', browser.url)
        self.assertNotIn('qwe', browser.contents)

    def test_technical_contacts_are_stored(self):
        ResourceGroup.create(name='qux', technical_contacts=['a', 'b'])
        browser = self.browser
        browser.open('http://localhost/resourcegroup/list')
        browser.getLink('qux').click()
        browser.getLink('Edit').click()
        self.assertEqual(
            'a', browser.getControl(name='contact', index=0).value)
        self.assertEqual(
            'b', browser.getControl(name='contact', index=1).value)
        browser.getControl(name='contact', index=0).value = 'foo'
        browser.getControl(name='save').click()
        self.assertIn('Updated on', browser.contents)
        self.assertIn('foo, b', browser.contents)

    def test_list_nodes_inside_resourcegroup(self):
        rg1 = ResourceGroup.create(name='rg1')
        rg2 = ResourceGroup.create(name='rg2')
        PhysicalMachine.create(
            name='test1', location=self.layer['loc0'], resource_group=rg1)
        PhysicalMachine.create(
            name='test2', location=self.layer['loc0'], resource_group=rg2)
        browser = self.browser
        browser.open('http://localhost/resourcegroup/list')
        browser.getLink('rg1').click()
        browser.getLink('Nodes').click()
        self.assertIn('test1', browser.contents)
        self.assertNotIn('test2', browser.contents)
