from ...location import Location
from ...machine import PhysicalMachine, VirtualMachine
from ...network import Network
from ...resourcegroup import ResourceGroup
import gocept.directory2.api.testing
import transaction


class TestRing1(gocept.directory2.api.testing.XMLRPCTestCase):

    def setUp(self):
        super(TestRing1, self).setUp()

        self.newyork = newyork = Location.create(abbreviation='ny')

        for vlan in newyork.vlans.values():
            Network.create(
                network='231.20.%s.0/24' % vlan.vlanclass.number,
                vlan=vlan)

            if vlan.name != 'ipmi':
                Network.create(
                    network='2002:470:9aaf:4%s::/64' % vlan.vlanclass.number,
                    vlan=vlan)

        jamaica = Location.create(abbreviation='jamaica')
        for vlan in jamaica.vlans.values():
            if vlan.name in ['ipmi', 'mgm', 'fe', 'srv']:
                Network.create(
                    network='172.20.%s.0/24' % vlan.vlanclass.number,
                    vlan=vlan)
            if vlan.name in ['sto', 'srv', 'fe']:
                Network.create(
                    network='2001:470:9aaf:4%s::/64' % vlan.vlanclass.number,
                    vlan=vlan)

        self.services = services = ResourceGroup.query().filter(
            ResourceGroup.name == 'services').one()
        services.timezone = 'UTC'
        self.stampy = stampy = PhysicalMachine.create(
            name='stampy',
            location=newyork,
            resource_group=services,
            directory_ring=0,
            directory_password='gfhdjk',
            service_description='backup server')

        mgm = stampy.nics['mgm']
        mgm.mac = '00:1e:c9:ad:4a:a6'
        newyork.vlans['mgm'].networks['231.20.1.0/24'].allocate(mgm)

        ipmi = stampy.nics['ipmi']
        ipmi.mac = '00:1e:c9:ad:4a:a7'
        newyork.vlans['ipmi'].networks['231.20.1.0/24'].allocate(ipmi)

        fe = stampy.nics['fe']
        fe.mac = '00:15:17:91:d2:f0'
        newyork.vlans['fe'].networks['231.20.2.0/24'].allocate(fe)

        srv = stampy.nics['srv']
        srv.mac = '00:1e:c9:ad:4a:a0'
        newyork.vlans['srv'].networks['231.20.3.0/24'].allocate(srv)

        sto = stampy.nics['sto']
        sto.mac = '00:1e:c9:ad:4a:a2'
        newyork.vlans['sto'].networks['231.20.4.0/24'].allocate(sto)

        jamaica.vlans['srv'].networks[
            '172.20.3.0/24'].dhcp_dynamic_range = True

        self.test00 = VirtualMachine.create(
            directory_ring=1,
            directory_password=None,
            service_description='test VM',
            name='test00',
            location=newyork,
            resource_group=services,
            memory=512,
            disk=5)

        transaction.commit()

        self.api = self.xmlrpc('/directory/services')

    def test_lookup_node(self):
        expected = {
            'classes': [''],
            'name': 'test00',
            'parameters': {
                'cores': None,
                'disk': 5,
                'environment': None,
                'id': 4101,
                'interfaces':
                    {'fe': {'mac': '02:00:00:02:10:05',
                            'networks': {
                                '2002:470:9aaf:42::/64':
                                ['2002:470:9aaf:42::3'],
                                '231.20.2.0/24':
                                []}},
                     'srv': {'mac': '02:00:00:03:10:05',
                             'networks': {
                                 '2002:470:9aaf:43::/64':
                                    ['2002:470:9aaf:43::3'],
                                 '231.20.3.0/24':
                                    []}}},
                'kvm_host': '',
                'location': 'ny',
                'mac_fe': '02:00:00:02:10:05',
                'mac_srv': '02:00:00:03:10:05',
                'machine': 'virtual',
                'memory': 512,
                'resource_group': 'services',
                'service_description': 'test VM',
                'timezone': 'UTC'}}
        self.assertEqual(expected, self.api.lookup_node('test00'))
