"""
Database utility providing access to the old "Domainshuttle" MySQL database.
"""
import sqlalchemy.ext.declarative
import sqlalchemy.orm
import zope.component
import zope.interface
import zope.sqlalchemy
import transaction
import gocept.directory2.db


class IDatabase(zope.interface.Interface):
    pass


class Database(gocept.directory2.db.Database):

    zope.interface.implementsOnly(IDatabase)

    def __init__(self, dsn, engine_args={}, testing=False):
        self.dsn = dsn
        self.testing = testing
        self._verify()
        if 'connect_args' not in engine_args:
            engine_args['connect_args'] = {}
        engine_args['connect_args'].update(dict(charset='utf8'))
        self.engine = sqlalchemy.create_engine(dsn, **engine_args)
        self.session_factory = sqlalchemy.orm.scoped_session(
            sqlalchemy.orm.sessionmaker(
                bind=self.engine,
                extension=zope.sqlalchemy.ZopeTransactionExtension(
                    keep_session=testing)))

    @staticmethod
    def db():
        return zope.component.getUtility(IDatabase)

    def initialize_database(self):
        from .domain import Domain
        try:
            with transaction.manager:
                Domain.query().all()
        except sqlalchemy.exc.ProgrammingError:
            pass
        else:
            raise ValueError("Database already exists. "
                             "Refusing to initialize again.")
        Object.metadata.create_all(self.engine)


class Added(zope.interface.interfaces.ObjectEvent):
    """An object has been created and added to the session."""

    def __init__(self, object):
        self.object = object


class ObjectBase(gocept.directory2.db.ObjectBase):

    @staticmethod
    def db():
        return zope.component.getUtility(IDatabase)


Object = sqlalchemy.ext.declarative.declarative_base(cls=ObjectBase)
