from ..location import Location
from ..machine import Node
from ..resourcegroup import ResourceGroup
from pyramid_mailer.interfaces import IMailer
import gocept.directory2.testing
import re
import unittest
import zope.component


class LocationUnitTests(unittest.TestCase):

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    def _makeOne(self):
        return Location()

    def test_mail_contacts(self):
        services = ResourceGroup.query().filter(
            ResourceGroup.name == 'services').one()
        services.technical_contacts = ['bob@example.com']

        location = Location.create(name='whq')
        Node.create(name='foo',
                    resource_group=services,
                    location=location)

        location.mail_contacts(
            'Everything exploded',
            'Unfortunately the world ended and we have to suspend service. :(')
        mailer = zope.component.getUtility(IMailer)
        self.assertEquals(2, len(mailer.outbox))

        # The last mail is our carbon copy to admin@gocept.com
        mail = mailer.outbox[-1]
        mail_message = mail.to_message().as_string()
        self.assertEquals('gocept support <support@gocept.com>', mail.sender)
        self.assertEquals(['admin@gocept.com'], mail.recipients)
        self.assertTrue(re.search(r'\nDate: [^\n]+ \+0000\n', mail_message))
        self.assertTrue('Auto-Submitted: auto-generated\n' in mail_message)
        self.assertTrue('Precedence: bulk\n' in mail_message)
        self.assertTrue('Subject: [whq] Everything exploded\n' in mail_message)
        self.assertTrue('these nodes: (informational copy)' in mail_message)
        self.assertTrue('Hello,\n' in mail_message)
        self.assertTrue('-- \n' in mail_message)

        mail = mailer.outbox[-2]
        mail_message = mail.to_message().as_string()
        self.assertEquals('gocept support <support@gocept.com>', mail.sender)
        self.assertEquals(['bob@example.com'], mail.recipients)
        self.assertTrue(re.search(r'\nDate: [^\n]+ \+0000\n', mail_message))
        self.assertTrue('Auto-Submitted: auto-generated\n' in mail_message)
        self.assertTrue('Precedence: bulk\n' in mail_message)
        self.assertTrue('Subject: [whq] Everything exploded\n' in mail_message)
        self.assertTrue('Hello,\n' in mail_message)
        self.assertTrue('these nodes: foo' in mail_message)
        self.assertTrue('-- \n' in mail_message)
