from sqlalchemy import Column, Integer, BigInteger, Date, ForeignKey
from sqlalchemy import UniqueConstraint, String
import sqlalchemy.orm
import gocept.directory2.db


class Traffic(gocept.directory2.db.Object):

    __tablename__ = 'traffic'
    __table_args__ = (
        UniqueConstraint('resource_group_id', 'ipaddress', 'day'),)

    resource_group_id = Column(
        Integer, ForeignKey('resourcegroups.id'), nullable=False)
    resource_group = sqlalchemy.orm.relation(
        'ResourceGroup')

    ipaddress = Column(String)
    day = Column(Date)
    traffic = Column(BigInteger)

    @classmethod
    def store(cls, timestamp, ip, traffic):
        day = timestamp.date()
        record = Traffic.query().filter(
            Traffic.day == day,
            Traffic.ipaddress == ip.address,
            Traffic.resource_group == ip.nic.node.resource_group).first()
        if record is not None:
            record.traffic += traffic
        else:
            Traffic.create(
                day=day,
                ipaddress=ip.address,
                resource_group=ip.nic.node.resource_group,
                traffic=traffic)
