from .location import Location
from sqlalchemy import Column, String, Integer, ForeignKey
from sqlalchemy.orm import relation
from sqlalchemy.orm.collections import attribute_mapped_collection
import gocept.directory2.db
import grokcore.component as grok
import sqlalchemy.orm


class VLANClass(gocept.directory2.db.Object):

    __tablename__ = 'vlanclasses'

    name = Column(String, unique=True)
    abbreviation = Column(String)
    description = Column(String)
    number = Column(Integer)
    color = Column(String)      # Actually we've coded this into the CSS ... :/

    @classmethod
    def create_default_vlan_classes(cls):
        cls.create(
            name='Management',
            abbreviation='mgm',
            description='BMC, Switches, ...',
            number=1,
            color='#DDD'),

        cls.create(
            name='Frontend',
            abbreviation='fe',
            description='von aussen',
            number=2,
            color='#00FF00'),

        cls.create(
            name='Servernetz/Backend',
            abbreviation='srv',
            description='Cluster-interne Kommunikation',
            number=3,
            color='#FF0000'),

        cls.create(
            name='Storage',
            abbreviation='sto',
            description='SAN (iSCSI)',
            number=4,
            color='#000000'),

        cls.create(
            name='Workstation',
            abbreviation='ws',
            description='Arbeitsplatzrechner, WLAN',
            number=5,
            color='#00FFFF'),

        cls.create(
            name='Transfer',
            abbreviation='tr',
            description='Router-Router-Kopplung',
            number=6,
            color='#0000FF'),

        cls.create(
            name='IPMI',
            abbreviation='ipmi',
            description='IPMI (Alias fuer MGM)',
            number=1,
            color='#DDD')


class VLAN(gocept.directory2.db.Object):

    __tablename__ = 'vlans'

    vlanclass_id = Column(
        Integer, ForeignKey('vlanclasses.id'), nullable=False)
    vlanclass = relation('VLANClass')

    location_id = Column(Integer, ForeignKey('locations.id'), nullable=False)
    location = relation(
        'Location',
        backref=sqlalchemy.orm.backref(
            'vlans', cascade="all, delete-orphan",
            collection_class=attribute_mapped_collection('name')))

    @property
    def name(self):
        return self.vlanclass.abbreviation


@grok.subscribe(Location, gocept.directory2.db.Added)
def create_default_vlans(location, event):
    """Create VLANs for all VLAN classes."""
    for vlanclass in VLANClass.query().all():
        vlan = VLAN.create()
        vlan.vlanclass = vlanclass
        vlan.location = location
